local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("WarningFrame")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")

local Party

local defaults = {
	profile = {
		Width = 180,
		DefaultColor = {0, 0.5, 1},
		DefaultSize = 46,
	}
}

for i, setting in pairs(RW.defaults.profile) do
	defaults.profile[i] = setting
end

local inset = defaults.profile.BorderSize > 2 and defaults.profile.Insets or 0
local backdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
end

function plugin:PluginEnable()
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_DefaultsChanged")
	self:RegisterMessage("RW_Warning")
	Party = RW:GetPlugin("Raid")
end

function plugin:PluginDisable()
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local s = self:GetEffectiveScale()
	plugin.db.profile.x = self:GetLeft() * s
	plugin.db.profile.y = self:GetBottom() * s
end

local function getAnchor()
	if plugin.anchor then return plugin.anchor end
	local db = plugin.db.profile
	local frame = RW:CreateAnchor(plugin:GetName(), UIParent, "Warning frame")
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	frame:ClearAllPoints()
	local s = frame:GetEffectiveScale()
	if db.x then
		frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
	else
		frame:SetPoint("CENTER", UIParent, "CENTER", -0, 70)
	end
	frame:SetWidth(db.Width)
	plugin.anchor = frame
	return frame
end

local function getWarningFrame()
	if plugin.warningFrame then return plugin.warningFrame end
	local db = plugin.db.profile
	local anchor = getAnchor()
	
	local frame = CreateFrame("Frame", "RW_WarningFrame")
	frame:SetPoint("BOTTOM", anchor, "TOP")
	frame:SetWidth(db.Width)
	frame:SetHeight(1)
	
	local text = frame:CreateFontString(frame:GetName().."String", "ARTWORK")
	text:SetPoint("BOTTOM", frame, "TOP")
	
	local icon = frame:CreateTexture(frame:GetName().."Icon", "ARTWORK")
	icon:SetPoint("RIGHT", text, "LEFT", -10, 0)
	
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.timer = self.timer - elapsed
		if self.timer >= 3 and self.timer <= 4 then
			LowHealthFrame:SetAlpha(self.timer - 3)
		elseif self.timer <= 0.1 then
			frame:Hide()
		elseif self.timer <= 2 then
			frame:SetAlpha(self.timer/2)
		end
	end)
	
	frame.text = text
	frame.icon = icon
	plugin.warningFrame = frame
	plugin:ApplySettings()
	return frame
end

function plugin:ApplySettings()
	local frame = getWarningFrame()
	local anchor = getAnchor()
	local db = self.db.profile
	
	frame.text:SetFont(LSM:Fetch("font", db.Font), db.DefaultSize, "OUTLINE")
	frame.text:SetText(frame.text:GetText() or "T")
	frame.text:SetTextColor(unpack(db.DefaultColor))
	frame.icon:SetWidth(frame.text:GetStringHeight())
	frame.icon:SetHeight(frame.text:GetStringHeight())
end

function plugin:RW_Warning(message, text, icon, color)
	local frame = getWarningFrame()
	local c = color or self.db.profile.DefaultColor
	frame.text:SetText(text)
	frame.text:SetTextColor(unpack(c))
	frame.icon:SetTexture(icon)
	frame:Show()
	frame:SetAlpha(1)
	frame.timer = 5
end

function plugin:RW_ConfigMode(message, start)
	if start then
		getAnchor():Show()
		getWarningFrame():Show()
		getWarningFrame():SetAlpha(1)
		getWarningFrame().timer = 11111111
		getWarningFrame().text:SetText("Warning Frame")
	else
		getAnchor():Hide()
		getWarningFrame():Hide()
		getWarningFrame().timer = 0
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if 	self.db.profile[option] and option == "Font" then
		self.db.profile[option] = value
		self:ApplySettings()
	end
end

function plugin:GetOptions()
	local option = {
		type = "group",
		name = "Warning Frame",
		set = function(info, value) 
			plugin.db.profile[info[#info]] = value
			plugin:ApplySettings()
		end,
		get = function(info) return plugin.db.profile[info[#info]] end,
		args = {
			DefaultSize = {
				type = "range",
				name = "Font size",
				min = 5,
				max = 50,
				step = 1
			},
			DefaultColor = {
				type = "color",
				name = "Default color",
				get = function() return unpack(plugin.db.profile.DefaultColor) end,
				set = function(info, r,g,b,a)
					local c = plugin.db.profile.DefaultColor
					c[1], c[2], c[3] = r, g, b
					plugin:ApplySettings()
				end,
			}
		}
	}
	
	return option
end
