local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Boss = RW.bossPrototype
local messageProt = {}
local mt = {__index = messageProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.ICON = {
	STAR	= 0x00000001,
	CIRCLE 	= 0x00000002,
	DIAMOND	= 0x00000004,
	TRIANGLE = 0x00000008,
	MOON 	= 0x00000010,
	SQUARE 	= 0x00000020,
	CROSS 	= 0x00000040,
	SKULL 	= 0x00000080,
}

local iconCoords = {
	STAR		= {0, 		0.25,	0, 		0.25},
	CIRCLE 		= {0.25,	0.5,	0,		0.25},
	DIAMOND		= {0.5, 	0.75,	0,		0.25},
	TRIANGLE 	= {0.75,	1,		0,		0.25},
	MOON 		= {0,		0.25,	0.25,	0.5},
	SQUARE 		= {0.25,	0.5,	0.25,	0.5},
	CROSS 		= {0.5,		0.75,	0.25,	0.5},
	SKULL 		= {0.75,	1,		0.25,	0.5},
}

function Boss:_Icon(defaultIcon, optionName)	
	obj = setmetatable({
		type = defaultIcon,
		icon = defaultIcon,
		id = optionName,
		scheduled = {},
		mod = self,
	}, mt)
	
	self.icons = self.icons or {}
	tinsert(self.icons, obj)
	return obj
end

function Boss:InsertIconOption(optionName, iType, default)
	local defaultOption = self.defaultOptions[optionName]
	local option = self.options[optionName]
	-- Add the default state
	if ((default == nil) or default) then
		defaultOption.uses = RW:AddBit(defaultOption.uses, BITS.ICON)
		defaultOption.ICON = RW:AddBit(defaultOption.ICON, self.BITS.ICON[iType])
	else
		defaultOption.ICON = RW:AddBit(defaultOption.ICON, 0)
	end
	
	-- Add settings for so that UI can draw them no matter if they are default on or off
	option.ICON = RW:AddBit(option.ICON, self.BITS.ICON[iType])
	option.uses = RW:AddBit(option.uses, BITS.ICON)
	
	self.usedIcons = self.usedIcons or {}
	self.usedIcons[iType] = true
end

function Boss:Icon(defaultIcon, optionName, default, ...)
	if not optionName then optionName = "Default" end
	if type(optionName) == "number" then
		self:AddSpellOption(optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName)
	end
	
	for i = 1, select("#", ...) do
		local n = select(i, ...)
		if type(n) == "string" then
			self:InsertIconOption(optionName, n, default)
		elseif type(n) == "boolean" then
			default = n
		end
	end
	self:InsertIconOption(optionName, defaultIcon, default)
	return self:_Icon(defaultIcon, optionName)
end

function messageProt:Show(target, time, icon, ...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.ICON) ~= RW.BITS.ICON then return end
	if bit.band(self.mod.db.profile[self.id].ICON, self.mod.BITS.ICON[icon or self.type]) ~= self.mod.BITS.ICON[icon or self.type] then return end

	local targetID
	if target:find("^0x") then	
		targetID = Boss:GetRaidTargetFromGUID(target)
	else
	end
	
	if targetID then
		self.oldIcon = GetRaidTargetIndex(targetID)
		SetRaidTarget(targetID, icon or self.icon or 1)
		if time or self.timeOut then
			--RW.ScheduleTimer(self.mod, "SetIcon", time or self.timeOut, {targetID, self.oldIcon or 0})
		end
	end
end
messageProt.Start = messageProt.Show

function messageProt:Schedule(stime, info, ...)
	self.mod:Schedule(self.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function messageProt:ScheduleRepeating(stime, now, info, ...)
	self.mod:ScheduleRepeating(self.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function messageProt:Unschedule(extraID)
	self.mod:Unschedule(self.id..(extraID or ""))
	RW:TRemove(self.scheduled, self.id..(extraID or ""))
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.ICON(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")	
	group:SetLayout("Flow")
	
	for bType, value in pairs(self.BITS.ICON) do
		if bit.band(self.options[option.text].ICON, value) == value then
			local check = AceGUI:Create("ImageCheckBox")
			check:SetRelativeWidth(0.2)
			check:SetValue(bit.band(self.db.profile[option.text].ICON, value) == value)
			check:SetUserData("option", option)
			check:SetUserData("type", "ICON")
			check:SetUserData("subType", bType)
			check:SetUserData("boss", self)
			check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)	
			check:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[bType]))
			group:AddChild(check)
		end
	end
	return group
end 