local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Boss = RW.bossPrototype
local messageProt = {}
local mt = {__index = messageProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.MESSAGE = {
	NORMAL	= 0x00000001,
	CAST 	= 0x00000002,
	TARGET 	= 0x00000004,
	TARGET_STACK	= 0x00000008,
	PHASE	= 0x00000010,
}

local custText = true
function Boss:_Message(mType, text, icon, optionName, default, color, customText)
	if type(text) == "number" then 
		custText = customText or false
		return self:_Message(mType, GetSpellInfo(text), text, text, icon, optionName) 
	end
	if type(icon) == "number" then return self:_Message(mType, text, select(3, GetSpellInfo(icon)), optionName, default, color) end
	
	if not optionName then optionName = text end
	if type(optionName) == "number" then
		self:AddSpellOption(optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName)
	end
	
	local defaultOption = self.defaultOptions[optionName]
	local option = self.options[optionName]
	
	-- Add the default state
	if ((default == nil) or default) then
		defaultOption.uses = RW:AddBit(defaultOption.uses, BITS.MESSAGE)
		defaultOption.MESSAGE = RW:AddBit(defaultOption.MESSAGE, self.BITS.MESSAGE[mType])
	else
		defaultOption.MESSAGE = RW:AddBit(defaultOption.MESSAGE, 0)
	end
	defaultOption.endMessage = endMessage or false
	-- Add settings for so that UI can draw them no matter if they are default on or off
	option.MESSAGE = RW:AddBit(option.MESSAGE, self.BITS.MESSAGE[mType])
	option.uses = RW:AddBit(option.uses, BITS.MESSAGE)
	
	obj = setmetatable({
		type = mType,
		text = text,
		customText = custText,
		icon = icon,
		id = optionName,
		color = color,
		scheduled = {},
		mod = self,
	}, mt)
	
	self.messages = self.messages or {}
	tinsert(self.messages, obj)
	
	custText = true
	
	return obj
end

function Boss:Message(...)
	return self:_Message("NORMAL", ...)
end

function Boss:CastMessage(time, ...)
	return self:_Message("CAST", time, ...)
end

function Boss:TargetMessage(...)
	return self:_Message("TARGET", ...)
end

function Boss:TargetStackMessage(...)
	return self:_Message("TARGET_STACK", ...)
end

function Boss:PhaseMessage(...)
	return self:_Message("PHASE", ...)
end

function messageProt:Show(...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.MESSAGE) ~= RW.BITS.MESSAGE then return end
	if bit.band(self.mod.db.profile[self.id].MESSAGE, self.mod.BITS.MESSAGE[self.type]) ~= self.mod.BITS.MESSAGE[self.type] then return end
	
	local text
	if self.customText then		
		text = RW:Format(self.text, ...)
	else
		text = RW:Format(L["FORMAT_MESSAGE_"..self.type], self.text, ...)
	end
	text = text:gsub(">([%S\s]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class] or self.color
			name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
		end
		return name
	end)
	
	local c = self.color or {r=1, g=1, b=1}
	self.mod:SendMessage("RW_Message", text, c.r, c.g, c.b, self.icon)
	self.mod:Msg((self.icon and "|T"..self.icon..":20|t " or "")..text)
end
messageProt.Start = messageProt.Show

local function showMessage(message, ...)
	message:Show(...)
end

function messageProt:Schedule(stime, info, ...)
	self.mod:Schedule(self.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function messageProt:ScheduleRepeating(stime, now, info, ...)
	self.mod:ScheduleRepeating(self.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function messageProt:Unschedule(extraID)
	self.mod:Unschedule(self.id..(extraID or ""))
	RW:TRemove(self.scheduled, self.id..(extraID or ""))
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.MESSAGE(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")
	
	for bType, value in pairs(self.BITS.MESSAGE) do
		if bit.band(self.options[option.text].MESSAGE, value) == value then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel(L["MESSAGE_"..bType]) -- TODO: Set more info in the text, (the actual time for the MESSAGE)
			check:SetValue(bit.band(self.db.profile[option.text].MESSAGE, value) == value)
			check:SetUserData("option", option)
			check:SetUserData("type", "MESSAGE")
			check:SetUserData("subType", bType)
			check:SetUserData("boss", self)
			check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)
			check:SetCallback("OnEnter", self.optionHelpers.onOptionEnter)
			check:SetCallback("OnLeave", self.optionHelpers.onOptionLeave)
			group:AddChild(check)
		end
	end
	return group
end 